/*
  
  	wmi8k by Martin Stoffel (martin@stoffel.net)
  
	Based on wmmon
	
	[Orig wmmon comments]

	Code based on wmppp/wmifs

	[Orig WMPPP comments]

	This code was mainly put together by looking at the
	following programs:

	asclock
		A neat piece of equip, used to display the date
		and time on the screen.
		Comes with every AfterStep installation.

		Source used:
			How do I create a not so solid window?
			How do I open a window?
			How do I use pixmaps?
	
	------------------------------------------------------------

	Authors: Martijn Pieterse (pieterse@xs4all.nl)
		 Antoine Nulle (warp@xs4all.nl)

	This program is distributed under the GPL license.
	(as were asclock and pppstats)

	----
	Changes:
	----
*/

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>

#include <sys/wait.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/ioctl.h>

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>

#include "../wmgeneral/wmgeneral.h"
#include "../wmgeneral/misc.h"


/* THIS NEEDS TO BE FROM THE ACTUAL USED KERNEL MODULE */
#include "i8k.h"

#include "wmi8k-master.xpm"
#include "wmi8k-mask.xbm"

  /***********/
 /* Defines */
/***********/

#define LEFT_ACTION (NULL)
#define RIGHT_ACTION (NULL)
#define MIDDLE_ACTION (NULL)

#define wmi8k_VERSION "1.0"

  /********************/
 /* Global Variables */
/********************/

int	stat_current = 0; /* now global */


/* functions */
void wmi8k_routine(int, char **);

static int i8k_fd;

/* the next three functions come from i8k utility package */

int i8k_init(){
    i8k_fd = open(I8K_PROC, O_RDONLY);
    if (i8k_fd < 0) {
	perror("can't open " I8K_PROC);
	exit(1);
    }
    return 0;
}

int i8k_get_fan_speed(int fan){
    int args[1];
    int rc;

    args[0] = fan;
    if ((rc=ioctl(i8k_fd, I8K_GET_SPEED, &args)) < 0) {
	return rc;
    }

    return args[0];
}

int i8k_get_cpu_temp(){
    int args[1];
    int rc;

    if ((rc=ioctl(i8k_fd, I8K_GET_TEMP, &args)) < 0) {
	return rc;
    }

    return args[0];
}


int i8k_set_fan(int fan, int speed)
{
    int args[2];
    int rc;

    args[0] = fan;
    args[1] = speed;
    if ((rc=ioctl(i8k_fd, I8K_SET_FAN, &args)) < 0) {
	return rc;
    }

    return args[0];
}

int i8k_get_fan_status(int fan)
{
    int args[1];
    int rc;

    args[0] = fan;
    if ((rc=ioctl(i8k_fd, I8K_GET_FAN, &args)) < 0) {
	return rc;
    }

    return args[0];
}




int main(int argc, char *argv[]) {
	/*init i8k */
	i8k_init();
	
	/* start main routine */
	wmi8k_routine(argc, argv);
	return 0;
}

/*******************************************************************************\
|* wmi8k_routine					     		       *|
\*******************************************************************************/


char		*left_action;
char		*right_action;
char		*middle_action;




void wmi8k_routine(int argc, char **argv) {


	int	i,j,k;
	XEvent	Event;
	int	but_stat = -1;


	
	int 	stat_max=2;
	int     actPos=-1;
	const int maxPos=55;
	int    his[55];
	int    hisR[55],hisL[55];

	int	skip=0;

	int	xpm_X = 0, xpm_Y = 0;



	if (LEFT_ACTION) left_action = strdup(LEFT_ACTION);
	if (RIGHT_ACTION) right_action = strdup(RIGHT_ACTION);
	if (MIDDLE_ACTION) middle_action = strdup(MIDDLE_ACTION);

	/* set the history to 0 */
	for(i=0;i<maxPos;i++){
		his[i]=0;
		hisL[i]=0;
		hisR[i]=0;
	}


	openXwindow(argc, argv, wmi8k_master_xpm, wmi8k_mask_bits, wmi8k_mask_width, wmi8k_mask_height);

	/* add mouse region */
	AddMouseRegion(0, 5, 28, 58, 57);
	AddMouseRegion(1, 5 ,5 ,25 ,15);
	AddMouseRegion(2, 5 ,17 ,25 ,27);
	AddMouseRegion(3, 5, 5, 58, 57);

	while (1) {

		waitpid(0, NULL, WNOHANG);

		
			/* draw themerature graph */
			skip++;
			skip%=34;			
			if (skip==0){		 			 		 
				/* scale from 45 to 85 degrees */
				his[actPos]= (i8k_get_cpu_temp()-45.0)/1.25; 
				
				/* scale left and right fan speeds form 5000-10000rpm to 0-40*/
				hisL[actPos]=(i8k_get_fan_speed(I8K_FAN_LEFT)-5000.0)/125;
				if (hisL[actPos]>40) hisL[actPos]=40;
				else if (hisL[actPos]<0) hisL[actPos]=0;
				
				hisR[actPos]=(i8k_get_fan_speed(I8K_FAN_RIGHT)-5000.0)/125;
				if (hisR[actPos]>40) hisR[actPos]=40;
				else if (hisR[actPos]<0) hisR[actPos]=0;
				
				for(i=0;i<maxPos;i++){
				/* wrap i his?[] used as a circular memory */
				j=(i+1+actPos)%maxPos;
					for(k=0;k<his[j];k++)
				   		copyXPMArea(2+k, 93, 1, 1, i+4, 58-k);		
					copyXPMArea(2+k, 94, 1, 1, i+4, 58-k+1);			
					copyXPMArea(2+k, 94, 1, 1, i+4, 58-k+2);		
					for( ;k<41;k++)
						copyXPMArea(2, 90, 1, 1, i+4, 58-k);
				/* draw the fan lines */
				if(hisR[j]!=0){
					copyXPMArea(3, 90, 1, 1, i+4, 58-hisR[j]+1);
					copyXPMArea(3, 91, 1, 1, i+4, 58-hisR[j]);
				}
				if(hisL[j]!=0){
					copyXPMArea(4, 90, 1, 1, i+4, 58-hisL[j]-1);
					copyXPMArea(4, 91, 1, 1, i+4, 58-hisL[j]);
				}						
				}
				actPos++;
				actPos%=maxPos;		
				
				
				
				
			}
			i=i8k_get_cpu_temp();
				
			copyXPMArea(66 + (i % 10)*6, 66, 5, 7, 52, 6);
			i /=10;
			copyXPMArea(66 + (i % 10)*6, 66, 5, 7, 45, 6);		
			i/=10;
			if(i!=0)
				copyXPMArea(66 + (i % 10)*6, 66, 5, 7, 38, 6);


			/*clear fan xx */
			copyXPMArea(0,64,32,12,28+64,4);			 
			copyXPMArea(0,64,32,12,28+64,18); 
			/* between 5000 and 10000 */
			i = i8k_get_fan_speed(I8K_FAN_LEFT);
			j=(i-5000)*0.0064;			
			if(j>32) j=32;
			if(i==0) j=0;
			copyXPMArea(32,64,j,12,28+64,4);			 
			i = i8k_get_fan_speed(I8K_FAN_RIGHT);
			j=(i-5000.0)*0.0064;
			if(j>32) j=32;
			if(i==0) j=0;
			copyXPMArea(32,64,j,12,28+64,18); 
			
			
			if(skip==0){
			 /* writ the fan speeds */
			i = i8k_get_fan_speed(I8K_FAN_LEFT);			
			copyXPMArea(40 + (i % 10)*7, 78, 6, 9, 114, 35);
			i /=10;
			copyXPMArea(40 + (i % 10)*7, 78, 6, 9, 107, 35);		
			i/=10;
			copyXPMArea(40 + (i % 10)*7, 78, 6, 9, 100, 35);
			i /=10;
			copyXPMArea(40 + (i % 10)*7, 78, 6, 9, 93, 35);
			i /=10;
			copyXPMArea(40 + (i % 10)*7, 78, 6, 9, 86, 35);
			//lower
			i = i8k_get_fan_speed(I8K_FAN_RIGHT);			
			copyXPMArea(40 + (i % 10)*7, 78, 6, 9, 114, 47);
			i /=10;
			copyXPMArea(40 + (i % 10)*7, 78, 6, 9, 107, 47);
			i/=10;
			copyXPMArea(40 + (i % 10)*7, 78, 6, 9, 100, 47);
			i /=10;
			copyXPMArea(40 + (i % 10)*7, 78, 6, 9, 93, 47);
			i /=10;
			copyXPMArea(40 + (i % 10)*7, 78, 6, 9, 86, 47);
			
			}

		RedrawWindowXY(xpm_X, xpm_Y);
	/* mouse acations */
		while (XPending(display)) {
			XNextEvent(display, &Event);
			switch (Event.type) {
			case Expose:
				RedrawWindowXY(xpm_X, xpm_Y);
				break;
			case DestroyNotify:
				XCloseDisplay(display);
				exit(0);
				break;
			case ButtonPress:
				but_stat = CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
				break;
			case ButtonRelease:				
				i = CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
				if (but_stat == i && but_stat >= 0) {
					switch (but_stat) {
					case 0:
						switch (Event.xbutton.button) {
						case 1:
							if (left_action)
								execCommand(left_action);
							break;
						case 2:
							if (middle_action)
								execCommand(middle_action);
							break;
						case 3:
							if (right_action)
								execCommand(right_action);
							break;
						}		
					case 3:			
						stat_current++;
						stat_current%=stat_max;
						printf("current stat is :%d\n", stat_current);
						if (stat_current == 1) {
							xpm_X = 64;
							setMaskXY(-64, 0);
						} else {
							xpm_X = 0;
							setMaskXY(0, 0);
						}
						RedrawWindowXY(xpm_X, xpm_Y);
						break;
					case 1:
						if(stat_current==1)
							i8k_set_fan(1,(i8k_get_fan_status(1)+1)%3);
						break;
					case 2:
						if(stat_current==1)
							i8k_set_fan(0,(i8k_get_fan_status(0)+1)%3);
						break;	
							
						
					}
				}
				break;
			}
		}

		usleep(250000L);
	}
}

